@@ -36,12 +36,14 @@ class BaseMeta(BaseAttack):
         whether to attack graph structure
     attack_features : bool
         whether to attack node features
+    undirected : bool
+        whether the graph is undirected
     device: str
         'cpu' or 'cuda'
 
     """
 
-    def __init__(self, model=None, nnodes=None, feature_shape=None, lambda_=0.5, attack_structure=True, attack_features=False, device='cpu'):
+    def __init__(self, model=None, nnodes=None, feature_shape=None, lambda_=0.5, attack_structure=True, attack_features=False, undirected=True, device='cpu'):
 
         super(BaseMeta, self).__init__(model, nnodes, attack_structure, attack_features, device)
         self.lambda_ = lambda_
@@ -52,6 +54,7 @@ class BaseMeta(BaseAttack):
         self.modified_features = None
 
         if attack_structure:
+            self.undirected = undirected
             assert nnodes is not None, 'Please give nnodes='
             self.adj_changes = Parameter(torch.FloatTensor(nnodes, nnodes))
             self.adj_changes.data.fill_(0)
@@ -68,9 +71,11 @@ class BaseMeta(BaseAttack):
 
     def get_modified_adj(self, ori_adj):
         adj_changes_square = self.adj_changes - torch.diag(torch.diag(self.adj_changes, 0))
-        ind = np.diag_indices(self.adj_changes.shape[0])
-        adj_changes_symm = torch.clamp(adj_changes_square + torch.transpose(adj_changes_square, 1, 0), -1, 1)
-        modified_adj = adj_changes_symm + ori_adj
+        # ind = np.diag_indices(self.adj_changes.shape[0]) # this line seems useless
+        if self.undirected:
+            adj_changes_square = adj_changes_square + torch.transpose(adj_changes_square, 1, 0)
+        adj_changes_square = torch.clamp(adj_changes_square, -1, 1)
+        modified_adj = adj_changes_square + ori_adj
         return modified_adj
 
     def get_modified_features(self, ori_features):
@@ -86,8 +91,9 @@ class BaseMeta(BaseAttack):
         degree_one = (degrees == 1)
         resh = degree_one.repeat(modified_adj.shape[0], 1).float()
         l_and = resh * modified_adj
-        logical_and_symmetric = l_and + l_and.t()
-        flat_mask = 1 - logical_and_symmetric
+        if self.undirected:
+            l_and = l_and + l_and.t()
+        flat_mask = 1 - l_and
         return flat_mask
 
     def self_training_label(self, labels, idx_train):
@@ -106,11 +112,14 @@ class BaseMeta(BaseAttack):
         Note that different data type (float, double) can effect the final results.
         """
         t_d_min = torch.tensor(2.0).to(self.device)
-        t_possible_edges = np.array(np.triu(np.ones((self.nnodes, self.nnodes)), k=1).nonzero()).T
+        if self.undirected:
+            t_possible_edges = np.array(np.triu(np.ones((self.nnodes, self.nnodes)), k=1).nonzero()).T
+        else:
+            t_possible_edges = np.array((np.ones((self.nnodes, self.nnodes)) - np.eye(self.nnodes)).nonzero()).T
         allowed_mask, current_ratio = utils.likelihood_ratio_filter(t_possible_edges,
                                                                     modified_adj,
                                                                     ori_adj, t_d_min,
-                                                                    ll_cutoff)
+                                                                    ll_cutoff, undirected=self.undirected)
         return allowed_mask, current_ratio
 
     def get_adj_score(self, adj_grad, modified_adj, ori_adj, ll_constraint, ll_cutoff):
@@ -164,9 +173,9 @@ class Metattack(BaseMeta):
 
     """
 
-    def __init__(self, model, nnodes, feature_shape=None, attack_structure=True, attack_features=False, device='cpu', with_bias=False, lambda_=0.5, train_iters=100, lr=0.1, momentum=0.9):
+    def __init__(self, model, nnodes, feature_shape=None, attack_structure=True, attack_features=False, undirected=True, device='cpu', with_bias=False, lambda_=0.5, train_iters=100, lr=0.1, momentum=0.9):
 
-        super(Metattack, self).__init__(model, nnodes, feature_shape, lambda_, attack_structure, attack_features, device)
+        super(Metattack, self).__init__(model, nnodes, feature_shape, lambda_, attack_structure, attack_features, undirected, device)
         self.momentum = momentum
         self.lr = lr
         self.train_iters = train_iters
@@ -354,7 +363,8 @@ class Metattack(BaseMeta):
                 adj_meta_argmax = torch.argmax(adj_meta_score)
                 row_idx, col_idx = utils.unravel_index(adj_meta_argmax, ori_adj.shape)
                 self.adj_changes.data[row_idx][col_idx] += (-2 * modified_adj[row_idx][col_idx] + 1)
-                self.adj_changes.data[col_idx][row_idx] += (-2 * modified_adj[row_idx][col_idx] + 1)
+                if self.undirected:
+                    self.adj_changes.data[col_idx][row_idx] += (-2 * modified_adj[row_idx][col_idx] + 1)
             else:
                 feature_meta_argmax = torch.argmax(feature_meta_score)
                 row_idx, col_idx = utils.unravel_index(feature_meta_argmax, ori_features.shape)
@@ -396,9 +406,9 @@ class MetaApprox(BaseMeta):
 
     """
 
-    def __init__(self, model, nnodes, feature_shape=None, attack_structure=True, attack_features=False, device='cpu', with_bias=False, lambda_=0.5, train_iters=100, lr=0.01):
+    def __init__(self, model, nnodes, feature_shape=None, attack_structure=True, attack_features=False, undirected=True, device='cpu', with_bias=False, lambda_=0.5, train_iters=100, lr=0.01):
 
-        super(MetaApprox, self).__init__(model, nnodes, feature_shape, lambda_, attack_structure, attack_features, device)
+        super(MetaApprox, self).__init__(model, nnodes, feature_shape, lambda_, attack_structure, attack_features, undirected, device)
 
         self.lr = lr
         self.train_iters = train_iters
@@ -549,7 +559,8 @@ class MetaApprox(BaseMeta):
                 adj_meta_argmax = torch.argmax(adj_meta_score)
                 row_idx, col_idx = utils.unravel_index(adj_meta_argmax, ori_adj.shape)
                 self.adj_changes.data[row_idx][col_idx] += (-2 * modified_adj[row_idx][col_idx] + 1)
-                self.adj_changes.data[col_idx][row_idx] += (-2 * modified_adj[row_idx][col_idx] + 1)
+                if self.undirected:
+                    self.adj_changes.data[col_idx][row_idx] += (-2 * modified_adj[row_idx][col_idx] + 1)
             else:
                 feature_meta_argmax = torch.argmax(feature_meta_score)
                 row_idx, col_idx = utils.unravel_index(feature_meta_argmax, ori_features.shape)
